# -*- ecoding: utf-8 -*-
# @ModuleName: __init__
# @Copyright: Deep_Wisdom 
# @Author: wk
# @Email: wangkai@fuzhi.ai
# @Time: 2022/1/20 8:21 下午

from .base_dataset import BaseDataset
from .emb_dataset import EmbDataset
from .process_data import get_dataloader
from .multi_task_dataset import MultiTaskDataset