# -*- ecoding: utf-8 -*-
# @ModuleName: __init__
# @Author: wk
# @Email: 306178200@qq.com
# @Time: 2022/6/10 7:40 PM
from .wdl import WDL
from .deepfm import DeepFM
from .nfm import NFM
from .fibinet import FiBiNet
from .afm import AFM
from .afn import AFN
from .aoanet import AOANet
from .autoint import AutoInt
from .ccpm import CCPM
from .lr import LR
from .fm import FM
from .xdeepfm import xDeepFM
from .dcn import DCN